@@ -22,11 +22,11 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef BLOCK_COPY_ON_READ
-#define BLOCK_COPY_ON_READ
+#ifndef BLOCK_COPY_ON_READ_H
+#define BLOCK_COPY_ON_READ_H
 
 #include "block/block_int.h"
 
 void bdrv_cor_filter_drop(BlockDriverState *cor_filter_bs);
 
-#endif /* BLOCK_COPY_ON_READ */
+#endif /* BLOCK_COPY_ON_READ_H */
