@@ -37,15 +37,8 @@
 
 // #define DEBUG_VERBOSE
 
-/* CURL 7.85.0 switches to a string based API for specifying
- * the desired protocols.
- */
-#if LIBCURL_VERSION_NUM >= 0x075500
-#define PROTOCOLS "HTTP,HTTPS,FTP,FTPS"
-#else
 #define PROTOCOLS (CURLPROTO_HTTP | CURLPROTO_HTTPS | \
                    CURLPROTO_FTP | CURLPROTO_FTPS)
-#endif
 
 #define CURL_NUM_STATES 8
 #define CURL_NUM_ACB    8
@@ -516,18 +509,9 @@ static int curl_init_state(BDRVCURLState *s, CURLState *state)
          * obscure protocols.  For example, do not allow POP3/SMTP/IMAP see
          * CVE-2013-0249.
          *
-         * Restricting protocols is only supported from 7.19.4 upwards. Note:
-         * version 7.85.0 deprecates CURLOPT_*PROTOCOLS in favour of a string
-         * based CURLOPT_*PROTOCOLS_STR API.
+         * Restricting protocols is only supported from 7.19.4 upwards.
          */
-#if LIBCURL_VERSION_NUM >= 0x075500
-        if (curl_easy_setopt(state->curl,
-                             CURLOPT_PROTOCOLS_STR, PROTOCOLS) ||
-            curl_easy_setopt(state->curl,
-                             CURLOPT_REDIR_PROTOCOLS_STR, PROTOCOLS)) {
-            goto err;
-        }
-#elif LIBCURL_VERSION_NUM >= 0x071304
+#if LIBCURL_VERSION_NUM >= 0x071304
         if (curl_easy_setopt(state->curl, CURLOPT_PROTOCOLS, PROTOCOLS) ||
             curl_easy_setopt(state->curl, CURLOPT_REDIR_PROTOCOLS, PROTOCOLS)) {
             goto err;
@@ -685,12 +669,7 @@ static int curl_open(BlockDriverState *bs, QDict *options, int flags,
     const char *file;
     const char *cookie;
     const char *cookie_secret;
-    /* CURL >= 7.55.0 uses curl_off_t for content length instead of a double */
-#if LIBCURL_VERSION_NUM >= 0x073700
-    curl_off_t cl;
-#else
-    double cl;
-#endif
+    double d;
     const char *secretid;
     const char *protocol_delimiter;
     int ret;
@@ -817,36 +796,27 @@ static int curl_open(BlockDriverState *bs, QDict *options, int flags,
     }
     if (curl_easy_perform(state->curl))
         goto out;
-    /* CURL 7.55.0 deprecates CURLINFO_CONTENT_LENGTH_DOWNLOAD in favour of
-     * the *_T version which returns a more sensible type for content length.
-     */
-#if LIBCURL_VERSION_NUM >= 0x073700
-    if (curl_easy_getinfo(state->curl, CURLINFO_CONTENT_LENGTH_DOWNLOAD_T, &cl)) {
+    if (curl_easy_getinfo(state->curl, CURLINFO_CONTENT_LENGTH_DOWNLOAD, &d)) {
         goto out;
     }
-#else
-    if (curl_easy_getinfo(state->curl, CURLINFO_CONTENT_LENGTH_DOWNLOAD, &cl)) {
-        goto out;
-    }
-#endif
     /* Prior CURL 7.19.4 return value of 0 could mean that the file size is not
      * know or the size is zero. From 7.19.4 CURL returns -1 if size is not
      * known and zero if it is really zero-length file. */
 #if LIBCURL_VERSION_NUM >= 0x071304
-    if (cl < 0) {
+    if (d < 0) {
         pstrcpy(state->errmsg, CURL_ERROR_SIZE,
                 "Server didn't report file size.");
         goto out;
     }
 #else
-    if (cl <= 0) {
+    if (d <= 0) {
         pstrcpy(state->errmsg, CURL_ERROR_SIZE,
                 "Unknown file size or zero-length file.");
         goto out;
     }
 #endif
 
-    s->len = cl;
+    s->len = d;
 
     if ((!strncasecmp(s->url, "http://", strlen("http://"))
         || !strncasecmp(s->url, "https://", strlen("https://")))
@@ -885,7 +855,7 @@ out_noclean:
     return -EINVAL;
 }
 
-static void curl_setup_preadv(BlockDriverState *bs, CURLAIOCB *acb)
+static void coroutine_fn curl_setup_preadv(BlockDriverState *bs, CURLAIOCB *acb)
 {
     CURLState *state;
     int running;
