@@ -270,9 +270,6 @@ static void xen_block_realize(XenDevice *xendev, Error **errp)
 
     xen_block_set_size(blockdev);
 
-    if (!monitor_add_blk(conf->blk, blockdev->drive->id, errp)) {
-        return;
-    }
     blockdev->dataplane =
         xen_block_dataplane_create(xendev, blk, conf->logical_block_size,
                                    blockdev->props.iothread);
@@ -725,8 +722,6 @@ static XenBlockDrive *xen_block_drive_create(const char *id,
     const char *mode = qdict_get_try_str(opts, "mode");
     const char *direct_io_safe = qdict_get_try_str(opts, "direct-io-safe");
     const char *discard_enable = qdict_get_try_str(opts, "discard-enable");
-    const char *suse_diskcache_disable_flush = qdict_get_try_str(opts,
-                                               "suse-diskcache-disable-flush");
     char *driver = NULL;
     char *filename = NULL;
     XenBlockDrive *drive = NULL;
@@ -807,16 +802,6 @@ static XenBlockDrive *xen_block_drive_create(const char *id,
         }
     }
 
-    if (suse_diskcache_disable_flush) {
-        unsigned long value;
-        if (!qemu_strtoul(suse_diskcache_disable_flush, NULL, 2, &value) && !!value) {
-            QDict *cache_qdict = qdict_new();
-
-            qdict_put_bool(cache_qdict, "no-flush", true);
-            qdict_put_obj(file_layer, "cache", QOBJECT(cache_qdict));
-        }
-    }
-
     /*
      * It is necessary to turn file locking off as an emulated device
      * may have already opened the same image file.
